@@ -23,8 +23,7 @@ from telegram import TelegramObject, ChatPhoto
 
 
 class Chat(TelegramObject):
-    """
-    This object represents a chat.
+    """This object represents a chat.
 
     Attributes:
         id (:obj:`int`): Unique identifier for this chat.
@@ -37,6 +36,8 @@ class Chat(TelegramObject):
         photo (:class:`telegram.ChatPhoto`): Optional. Chat photo.
         description (:obj:`str`): Optional. Description, for supergroups and channel chats.
         invite_link (:obj:`str`): Optional. Chat invite link, for supergroups and channel chats.
+        pinned_message (:class:`telegram.Message`): Optional. Pinned message, for supergroups.
+            Returned only in get_chat.
 
     Args:
         id (:obj:`int`): Unique identifier for this chat. This number may be greater than 32 bits
@@ -57,6 +58,8 @@ class Chat(TelegramObject):
             Returned only in get_chat.
         invite_link (:obj:`str`, optional): Chat invite link, for supergroups and channel chats.
             Returned only in get_chat.
+        pinned_message (:class:`telegram.Message`, optional): Pinned message, for supergroups.
+            Returned only in get_chat.
         bot (:class:`telegram.Bot`, optional): The Bot to use for instance methods.
         **kwargs (:obj:`dict`): Arbitrary keyword arguments.
 
@@ -83,6 +86,7 @@ class Chat(TelegramObject):
                  photo=None,
                  description=None,
                  invite_link=None,
+                 pinned_message=None,
                  **kwargs):
         # Required
         self.id = int(id)
@@ -96,6 +100,7 @@ class Chat(TelegramObject):
         self.photo = photo
         self.description = description
         self.invite_link = invite_link
+        self.pinned_message = pinned_message
 
         self.bot = bot
         self._id_attrs = (self.id,)
@@ -106,36 +111,36 @@ class Chat(TelegramObject):
             return None
 
         data['photo'] = ChatPhoto.de_json(data.get('photo'), bot)
+        from telegram import Message
+        data['pinned_message'] = Message.de_json(data.get('pinned_message'), bot)
 
         return cls(bot=bot, **data)
 
     def send_action(self, *args, **kwargs):
-        """
-        Shortcut for::
+        """Shortcut for::
 
             bot.send_chat_action(update.message.chat.id, *args, **kwargs)
 
         Returns:
             :obj:`bool`: If the action was sent successfully.
+
         """
 
         return self.bot.send_chat_action(self.id, *args, **kwargs)
 
     def leave(self, *args, **kwargs):
-        """
-        Shortcut for::
+        """Shortcut for::
 
             bot.leave_chat(update.message.chat.id, *args, **kwargs)
 
         Returns:
             :obj:`bool` If the action was sent successfully.
-        """
 
+        """
         return self.bot.leave_chat(self.id, *args, **kwargs)
 
     def get_administrators(self, *args, **kwargs):
-        """
-        Shortcut for::
+        """Shortcut for::
 
             bot.get_chat_administrators(update.message.chat.id, *args, **kwargs)
 
@@ -144,39 +149,36 @@ class Chat(TelegramObject):
             :class:`telegram.ChatMember` objects that contains information about all
             chat administrators except other bots. If the chat is a group or a supergroup
             and no administrators were appointed, only the creator will be returned
-        """
 
+        """
         return self.bot.get_chat_administrators(self.id, *args, **kwargs)
 
     def get_members_count(self, *args, **kwargs):
-        """
-        Shortcut for::
+        """Shortcut for::
 
             bot.get_chat_members_count(update.message.chat.id, *args, **kwargs)
 
         Returns:
             :obj:`int`
-        """
 
+        """
         return self.bot.get_chat_members_count(self.id, *args, **kwargs)
 
     def get_member(self, *args, **kwargs):
-        """
-        Shortcut for::
+        """Shortcut for::
 
             bot.get_chat_member(update.message.chat.id, *args, **kwargs)
 
         Returns:
             :class:`telegram.ChatMember`
-        """
 
+        """
         return self.bot.get_chat_member(self.id, *args, **kwargs)
 
     def kick_member(self, *args, **kwargs):
-        """
-        Shortcut for::
+        """Shortcut for::
 
-            bot.kick_chat_member(update.message.chat.id, *args, **kwargs)
+                bot.kick_chat_member(update.message.chat.id, *args, **kwargs)
 
         Returns:
             :obj:`bool`: If the action was sent succesfully.
@@ -185,17 +187,17 @@ class Chat(TelegramObject):
             This method will only work if the `All Members Are Admins` setting is off in the
             target group. Otherwise members may only be removed by the group's creator or by the
             member that added them.
-        """
 
+        """
         return self.bot.kick_chat_member(self.id, *args, **kwargs)
 
     def unban_member(self, *args, **kwargs):
-        """
-        Shortcut for::
+        """Shortcut for::
 
-            bot.unban_chat_member(update.message.chat.id, *args, **kwargs)
+                bot.unban_chat_member(update.message.chat.id, *args, **kwargs)
 
         Returns:
             :obj:`bool`: If the action was sent successfully.
+
         """
         return self.bot.unban_chat_member(self.id, *args, **kwargs)
