@@ -39,15 +39,15 @@ DEFAULT_GROUP = 0
 
 
 def run_async(func):
-    """
-    Function decorator that will run the function in a new thread.
-    will run :attr:`telegram.ext.Dispatcher.run_async`.
+    """Function decorator that will run the function in a new thread.
+
+    Will run :attr:`telegram.ext.Dispatcher.run_async`.
+
     Using this decorator is only possible when only a single Dispatcher exist in the system.
 
-    Note:
-        Use this decorator to run handlers asynchronously.
-    """
+    Note: Use this decorator to run handlers asynchronously.
 
+    """
     @wraps(func)
     def async_func(*args, **kwargs):
         return Dispatcher.get_instance().run_async(func, *args, **kwargs)
@@ -55,9 +55,13 @@ def run_async(func):
     return async_func
 
 
+class DispatcherHandlerStop(Exception):
+    """Raise this in handler to prevent execution any other handler (even in different group)."""
+    pass
+
+
 class Dispatcher(object):
-    """
-    This class dispatches all kinds of updates to its registered handlers.
+    """This class dispatches all kinds of updates to its registered handlers.
 
     Attributes:
         bot (:class:`telegram.Bot`): The bot object that should be passed to the handlers.
@@ -74,6 +78,7 @@ class Dispatcher(object):
                 instance to pass onto handler callbacks.
         workers (:obj:`int`, optional): Number of maximum concurrent worker threads for the
             ``@run_async`` decorator. defaults to 4.
+
     """
 
     __singleton_lock = Lock()
@@ -113,12 +118,6 @@ class Dispatcher(object):
             else:
                 self._set_singleton(None)
 
-    @classmethod
-    def _reset_singleton(cls):
-        # NOTE: This method was added mainly for test_updater benefit and specifically pypy. Never
-        #       call it in production code.
-        cls.__singleton_semaphore.release()
-
     def _init_async_threads(self, base_name, workers):
         base_name = '{}_'.format(base_name) if base_name else ''
 
@@ -134,16 +133,15 @@ class Dispatcher(object):
 
     @classmethod
     def get_instance(cls):
-        """
-        Get the singleton instance of this class.
+        """Get the singleton instance of this class.
 
         Returns:
             :class:`telegram.ext.Dispatcher`
 
         Raises:
             RuntimeError
-        """
 
+        """
         if cls.__singleton is not None:
             return cls.__singleton()
         else:
@@ -162,10 +160,13 @@ class Dispatcher(object):
                 break
 
             promise.run()
+            if isinstance(promise.exception, DispatcherHandlerStop):
+                self.logger.warning(
+                    'DispatcherHandlerStop is not supported with async functions; func: %s',
+                    promise.pooled_function.__name__)
 
     def run_async(self, func, *args, **kwargs):
-        """
-        Queue a function (with given args/kwargs) to be run asynchronously.
+        """Queue a function (with given args/kwargs) to be run asynchronously.
 
         Args:
             func (:obj:`callable`): The function to run in the thread.
@@ -183,11 +184,11 @@ class Dispatcher(object):
         return promise
 
     def start(self):
-        """
-        Thread target of thread 'dispatcher'. Runs in background and processes
-        the update queue.
-        """
+        """Thread target of thread 'dispatcher'.
 
+        Runs in background and processes the update queue.
+
+        """
         if self.running:
             self.logger.warning('already running')
             return
@@ -221,10 +222,7 @@ class Dispatcher(object):
         self.logger.debug('Dispatcher thread stopped')
 
     def stop(self):
-        """
-        Stops the thread.
-        """
-
+        """Stops the thread."""
         if self.running:
             self.__stop_event.set()
             while self.running:
@@ -251,64 +249,71 @@ class Dispatcher(object):
         return self.running or bool(self.__async_threads)
 
     def process_update(self, update):
-        """
-        Processes a single update.
+        """Processes a single update.
 
         Args:
-            update (:obj:`str` | :class:`telegram.Update`): The update to process.
-        """
+            update (:obj:`str` | :class:`telegram.Update` | :class:`telegram.TelegramError`):
+                The update to process.
 
+        """
         # An error happened while polling
         if isinstance(update, TelegramError):
-            self.dispatch_error(None, update)
+            try:
+                self.dispatch_error(None, update)
+            except Exception:
+                self.logger.exception('An uncaught error was raised while handling the error')
+            return
 
-        else:
-            for group in self.groups:
-                for handler in self.handlers[group]:
-                    try:
-                        if handler.check_update(update):
-                            handler.handle_update(update, self)
-                            break
-                    # Dispatch any errors
-                    except TelegramError as te:
-                        self.logger.warn('A TelegramError was raised while processing the '
-                                         'Update.')
+        for group in self.groups:
+            try:
+                for handler in (x for x in self.handlers[group] if x.check_update(update)):
+                    handler.handle_update(update, self)
+                    break
 
-                        try:
-                            self.dispatch_error(update, te)
-                        except Exception:
-                            self.logger.exception('An uncaught error was raised while '
-                                                  'handling the error')
-                        finally:
-                            break
+            # Stop processing with any other handler.
+            except DispatcherHandlerStop:
+                self.logger.debug('Stopping further handlers due to DispatcherHandlerStop')
+                break
 
-                    # Errors should not stop the thread
-                    except Exception:
-                        self.logger.exception('An uncaught error was raised while '
-                                              'processing the update')
-                        break
+            # Dispatch any error.
+            except TelegramError as te:
+                self.logger.warning('A TelegramError was raised while processing the Update')
+
+                try:
+                    self.dispatch_error(update, te)
+                except DispatcherHandlerStop:
+                    self.logger.debug('Error handler stopped further handlers')
+                    break
+                except Exception:
+                    self.logger.exception('An uncaught error was raised while handling the error')
+
+            # Errors should not stop the thread.
+            except Exception:
+                self.logger.exception('An uncaught error was raised while processing the update')
 
     def add_handler(self, handler, group=DEFAULT_GROUP):
-        """
-        Register a handler.
+        """Register a handler.
 
-        TL;DR: Order and priority counts. 0 or 1 handlers per group will be
-        used.
+        TL;DR: Order and priority counts. 0 or 1 handlers per group will be used.
 
         A handler must be an instance of a subclass of :class:`telegram.ext.Handler`. All handlers
         are organized in groups with a numeric value. The default group is 0. All groups will be
-        evaluated for handling an update, but only 0 or 1 handler per group will be used.
+        evaluated for handling an update, but only 0 or 1 handler per group will be used. If
+        :class:`telegram.DispatcherHandlerStop` is raised from one of the handlers, no further
+        handlers (regardless of the group) will be called.
 
         The priority/order of handlers is determined as follows:
 
           * Priority of the group (lower group number == higher priority)
-          * The first handler in a group which should handle an update will be
-            used. Other handlers from the group will not be used. The order in
-            which handlers were added to the group defines the priority.
+          * The first handler in a group which should handle an update (see
+            :attr:`telegram.ext.Handler.check_update`) will be used. Other handlers from the
+            group will not be used. The order in which handlers were added to the group defines the
+            priority.
 
         Args:
             handler (:class:`telegram.ext.Handler`): A Handler instance.
             group (:obj:`int`, optional): The group identifier. Default is 0.
+
         """
 
         if not isinstance(handler, Handler):
@@ -324,14 +329,13 @@ class Dispatcher(object):
         self.handlers[group].append(handler)
 
     def remove_handler(self, handler, group=DEFAULT_GROUP):
-        """
-        Remove a handler from the specified group
+        """Remove a handler from the specified group.
 
         Args:
             handler (:class:`telegram.ext.Handler`): A Handler instance.
             group (:obj:`object`, optional): The group identifier. Default is 0.
-        """
 
+        """
         if handler in self.handlers[group]:
             self.handlers[group].remove(handler)
             if not self.handlers[group]:
@@ -339,35 +343,32 @@ class Dispatcher(object):
                 self.groups.remove(group)
 
     def add_error_handler(self, callback):
-        """
-        Registers an error handler in the Dispatcher.
+        """Registers an error handler in the Dispatcher.
 
         Args:
-            handler (:obj:`callable`): A function that takes ``Bot, Update, TelegramError`` as
+            callback (:obj:`callable`): A function that takes ``Bot, Update, TelegramError`` as
                 arguments.
-        """
 
+        """
         self.error_handlers.append(callback)
 
     def remove_error_handler(self, callback):
-        """
-        Removes an error handler.
+        """Removes an error handler.
 
         Args:
-            handler (:obj:`callable`): The error handler to remove.
-        """
+            callback (:obj:`callable`): The error handler to remove.
 
+        """
         if callback in self.error_handlers:
             self.error_handlers.remove(callback)
 
     def dispatch_error(self, update, error):
-        """
-        Dispatches an error.
+        """Dispatches an error.
 
         Args:
-            update (:obj:`str` | :class:`telegram.Update`): The update that caused the error
+            update (:obj:`str` | :class:`telegram.Update` | None): The update that caused the error
             error (:class:`telegram.TelegramError`): The Telegram error that was raised.
-        """
 
+        """
         for callback in self.error_handlers:
             callback(self.bot, update, error)
