@@ -27,9 +27,11 @@ from telegram.utils.deprecate import warn_deprecate_obj
 from telegram.utils.helpers import escape_html, escape_markdown, to_timestamp, from_timestamp
 
 
+_UNDEFINED = object()
+
+
 class Message(TelegramObject):
-    """
-    This object represents a message.
+    """This object represents a message.
 
     Note:
         * In Python `from` is a reserved word, use `from_user` instead.
@@ -83,6 +85,10 @@ class Message(TelegramObject):
         invoice (:class:`telegram.Invoice`): Optional. Information about the invoice.
         successful_payment (:class:`telegram.SuccessfulPayment`): Optional. Information about the
             payment.
+        forward_signature (:obj:`str`): Optional. Signature of the post author for messages
+            forwarded from channels.
+        author_signature (:obj:`str`): Optional. Signature of the post author for messages
+            in channels.
         bot (:class:`telegram.Bot`): Optional. The Bot to use for instance methods.
 
     Deprecated: 6.0
@@ -169,8 +175,14 @@ class Message(TelegramObject):
             information about the invoice.
         successful_payment (:class:`telegram.SuccessfulPayment`, optional): Message is a service
             message about a successful payment, information about the payment.
+        forward_signature (:obj:`str`, optional): Signature of the post author for messages
+            forwarded from channels.
+        author_signature (:obj:`str`, optional): Signature of the post author for messages
+            in channels.
     """
 
+    _effective_attachment = _UNDEFINED
+
     def __init__(self,
                  message_id,
                  from_user,
@@ -210,6 +222,8 @@ class Message(TelegramObject):
                  pinned_message=None,
                  invoice=None,
                  successful_payment=None,
+                 forward_signature=None,
+                 author_signature=None,
                  bot=None,
                  **kwargs):
         # Required
@@ -228,7 +242,7 @@ class Message(TelegramObject):
         self.audio = audio
         self.game = game
         self.document = document
-        self.photo = photo
+        self.photo = photo or list()
         self.sticker = sticker
         self.video = video
         self.voice = voice
@@ -238,10 +252,10 @@ class Message(TelegramObject):
         self.location = location
         self.venue = venue
         self._new_chat_member = new_chat_member
-        self.new_chat_members = new_chat_members
+        self.new_chat_members = new_chat_members or list()
         self.left_chat_member = left_chat_member
         self.new_chat_title = new_chat_title
-        self.new_chat_photo = new_chat_photo
+        self.new_chat_photo = new_chat_photo or list()
         self.delete_chat_photo = bool(delete_chat_photo)
         self.group_chat_created = bool(group_chat_created)
         self.supergroup_chat_created = bool(supergroup_chat_created)
@@ -252,6 +266,8 @@ class Message(TelegramObject):
         self.forward_from_message_id = forward_from_message_id
         self.invoice = invoice
         self.successful_payment = successful_payment
+        self.forward_signature = forward_signature
+        self.author_signature = author_signature
 
         self.bot = bot
 
@@ -259,9 +275,7 @@ class Message(TelegramObject):
 
     @property
     def chat_id(self):
-        """
-        :obj:`int`: Shortcut for :attr:`telegram.Chat.id` for :attr:`chat`.
-        """
+        """:obj:`int`: Shortcut for :attr:`telegram.Chat.id` for :attr:`chat`."""
         return self.chat.id
 
     @classmethod
@@ -301,6 +315,39 @@ class Message(TelegramObject):
 
         return cls(bot=bot, **data)
 
+    @property
+    def effective_attachment(self):
+        """
+        :class:`telegram.Audio`
+            or :class:`telegram.Contact`
+            or :class:`telegram.Document`
+            or :class:`telegram.Game`
+            or :class:`telegram.Invoice`
+            or :class:`telegram.Location`
+            or List[:class:`telegram.PhotoSize`]
+            or :class:`telegram.Sticker`
+            or :class:`telegram.SuccessfulPayment`
+            or :class:`telegram.Venue`
+            or :class:`telegram.Video`
+            or :class:`telegram.VideoNote`
+            or :class:`telegram.Voice`: The attachment that this message was sent with. May be
+            ``None`` if no attachment was sent.
+
+        """
+        if self._effective_attachment is not _UNDEFINED:
+            return self._effective_attachment
+
+        for i in (self.audio, self.game, self.document, self.photo, self.sticker,
+                  self.video, self.voice, self.video_note, self.contact, self.location,
+                  self.venue, self.invoice, self.successful_payment):
+            if i:
+                self._effective_attachment = i
+                break
+        else:
+            self._effective_attachment = None
+
+        return self._effective_attachment
+
     def __getitem__(self, item):
         if item in self.__dict__.keys():
             return self.__dict__[item]
@@ -331,7 +378,7 @@ class Message(TelegramObject):
         return data
 
     def _quote(self, kwargs):
-        """Modify kwargs for replying with or without quoting"""
+        """Modify kwargs for replying with or without quoting."""
         if 'reply_to_message_id' in kwargs:
             if 'quote' in kwargs:
                 del kwargs['quote']
@@ -347,8 +394,7 @@ class Message(TelegramObject):
                 kwargs['reply_to_message_id'] = self.message_id
 
     def reply_text(self, *args, **kwargs):
-        """
-        Shortcut for::
+        """Shortcut for::
 
             bot.send_message(update.message.chat_id, *args, **kwargs)
 
@@ -357,14 +403,13 @@ class Message(TelegramObject):
                 reply to this message. If ``reply_to_message_id`` is passed in ``kwargs``, this
                 parameter will be ignored. Default: ``True`` in group chats and ``False`` in
                 private chats.
-        """
 
+        """
         self._quote(kwargs)
         return self.bot.send_message(self.chat_id, *args, **kwargs)
 
     def reply_photo(self, *args, **kwargs):
-        """
-        Shortcut for::
+        """Shortcut for::
 
             bot.send_photo(update.message.chat_id, *args, **kwargs)
 
@@ -375,14 +420,13 @@ class Message(TelegramObject):
 
         Returns:
             :class:`telegram.Message`: On success, instance representing the message posted.
-        """
 
+        """
         self._quote(kwargs)
         return self.bot.send_photo(self.chat_id, *args, **kwargs)
 
     def reply_audio(self, *args, **kwargs):
-        """
-        Shortcut for::
+        """Shortcut for::
 
             bot.send_audio(update.message.chat_id, *args, **kwargs)
 
@@ -393,14 +437,13 @@ class Message(TelegramObject):
 
         Returns:
             :class:`telegram.Message`: On success, instance representing the message posted.
-        """
 
+        """
         self._quote(kwargs)
         return self.bot.send_audio(self.chat_id, *args, **kwargs)
 
     def reply_document(self, *args, **kwargs):
-        """
-        Shortcut for::
+        """Shortcut for::
 
             bot.send_document(update.message.chat_id, *args, **kwargs)
 
@@ -411,14 +454,13 @@ class Message(TelegramObject):
 
         Returns:
             :class:`telegram.Message`: On success, instance representing the message posted.
-        """
 
+        """
         self._quote(kwargs)
         return self.bot.send_document(self.chat_id, *args, **kwargs)
 
     def reply_sticker(self, *args, **kwargs):
-        """
-        Shortcut for::
+        """Shortcut for::
 
             bot.send_sticker(update.message.chat_id, *args, **kwargs)
 
@@ -429,16 +471,15 @@ class Message(TelegramObject):
 
         Returns:
             :class:`telegram.Message`: On success, instance representing the message posted.
-        """
 
+        """
         self._quote(kwargs)
         return self.bot.send_sticker(self.chat_id, *args, **kwargs)
 
     def reply_video(self, *args, **kwargs):
-        """
-        Shortcut for::
+        """Shortcut for::
 
-                bot.send_video(update.message.chat_id, *args, **kwargs)
+            bot.send_video(update.message.chat_id, *args, **kwargs)
 
         Keyword Args:
             quote (:obj:`bool`, optional): If set to ``True``, the photo is sent as an actual reply
@@ -447,16 +488,15 @@ class Message(TelegramObject):
 
         Returns:
             :class:`telegram.Message`: On success, instance representing the message posted.
-        """
 
+        """
         self._quote(kwargs)
         return self.bot.send_video(self.chat_id, *args, **kwargs)
 
     def reply_video_note(self, *args, **kwargs):
-        """
-        Shortcut for::
+        """Shortcut for::
 
-                bot.send_video_note(update.message.chat_id, *args, **kwargs)
+            bot.send_video_note(update.message.chat_id, *args, **kwargs)
 
         Keyword Args:
             quote (:obj:`bool`, optional): If set to ``True``, the photo is sent as an actual reply
@@ -465,14 +505,13 @@ class Message(TelegramObject):
 
         Returns:
             :class:`telegram.Message`: On success, instance representing the message posted.
-        """
 
+        """
         self._quote(kwargs)
         return self.bot.send_video_note(self.chat_id, *args, **kwargs)
 
     def reply_voice(self, *args, **kwargs):
-        """
-        Shortcut for::
+        """Shortcut for::
 
             bot.send_voice(update.message.chat_id, *args, **kwargs)
 
@@ -483,16 +522,15 @@ class Message(TelegramObject):
 
         Returns:
             :class:`telegram.Message`: On success, instance representing the message posted.
-        """
 
+        """
         self._quote(kwargs)
         return self.bot.send_voice(self.chat_id, *args, **kwargs)
 
     def reply_location(self, *args, **kwargs):
-        """
-        Shortcut for::
+        """Shortcut for::
 
-                bot.send_location(update.message.chat_id, *args, **kwargs)
+            bot.send_location(update.message.chat_id, *args, **kwargs)
 
         Keyword Args:
             quote (:obj:`bool`, optional): If set to ``True``, the photo is sent as an actual reply
@@ -501,16 +539,15 @@ class Message(TelegramObject):
 
         Returns:
             :class:`telegram.Message`: On success, instance representing the message posted.
-        """
 
+        """
         self._quote(kwargs)
         return self.bot.send_location(self.chat_id, *args, **kwargs)
 
     def reply_venue(self, *args, **kwargs):
-        """
-        Shortcut for::
+        """Shortcut for::
 
-                bot.send_venue(update.message.chat_id, *args, **kwargs)
+            bot.send_venue(update.message.chat_id, *args, **kwargs)
 
         Keyword Args:
             quote (:obj:`bool`, optional): If set to ``True``, the photo is sent as an actual reply
@@ -519,16 +556,15 @@ class Message(TelegramObject):
 
         Returns:
             :class:`telegram.Message`: On success, instance representing the message posted.
-        """
 
+        """
         self._quote(kwargs)
         return self.bot.send_venue(self.chat_id, *args, **kwargs)
 
     def reply_contact(self, *args, **kwargs):
-        """
-        Shortcut for::
+        """Shortcut for::
 
-                bot.send_contact(update.message.chat_id, *args, **kwargs)
+            bot.send_contact(update.message.chat_id, *args, **kwargs)
 
         Keyword Args:
             quote (:obj:`bool`, optional): If set to ``True``, the photo is sent as an actual reply
@@ -537,24 +573,23 @@ class Message(TelegramObject):
 
         Returns:
             :class:`telegram.Message`: On success, instance representing the message posted.
-        """
 
+        """
         self._quote(kwargs)
         return self.bot.send_contact(self.chat_id, *args, **kwargs)
 
     def forward(self, chat_id, disable_notification=False):
-        """
-        Shortcut for::
+        """Shortcut for::
 
-                bot.forward_message(chat_id=chat_id,
-                                    from_chat_id=update.message.chat_id,
-                                    disable_notification=disable_notification,
-                                    message_id=update.message.message_id)
+            bot.forward_message(chat_id=chat_id,
+                                from_chat_id=update.message.chat_id,
+                                disable_notification=disable_notification,
+                                message_id=update.message.message_id)
 
         Returns:
             :class:`telegram.Message`: On success, instance representing the message forwarded.
-        """
 
+        """
         return self.bot.forward_message(
             chat_id=chat_id,
             from_chat_id=self.chat_id,
@@ -562,13 +597,12 @@ class Message(TelegramObject):
             message_id=self.message_id)
 
     def edit_text(self, *args, **kwargs):
-        """
-        Shortcut for::
+        """Shortcut for::
 
-                bot.edit_message_text(chat_id=message.chat_id,
-                                      message_id=message.message_id,
-                                      *args,
-                                      **kwargs)
+            bot.edit_message_text(chat_id=message.chat_id,
+                                  message_id=message.message_id,
+                                  *args,
+                                  **kwargs)
 
         Note:
             You can only edit messages that the bot sent itself,
@@ -577,19 +611,18 @@ class Message(TelegramObject):
 
         Returns:
             :class:`telegram.Message`: On success, instance representing the edited message.
-        """
 
+        """
         return self.bot.edit_message_text(
             chat_id=self.chat_id, message_id=self.message_id, *args, **kwargs)
 
     def edit_caption(self, *args, **kwargs):
-        """
-        Shortcut for::
+        """Shortcut for::
 
-                bot.edit_message_caption(chat_id=message.chat_id,
-                                         message_id=message.message_id,
-                                         *args,
-                                         **kwargs)
+            bot.edit_message_caption(chat_id=message.chat_id,
+                                     message_id=message.message_id,
+                                     *args,
+                                     **kwargs)
 
         Note:
             You can only edit messages that the bot sent itself,
@@ -598,19 +631,18 @@ class Message(TelegramObject):
 
         Returns:
             :class:`telegram.Message`: On success, instance representing the edited message.
-        """
 
+        """
         return self.bot.edit_message_caption(
             chat_id=self.chat_id, message_id=self.message_id, *args, **kwargs)
 
     def edit_reply_markup(self, *args, **kwargs):
-        """
-        Shortcut for::
+        """Shortcut for::
 
-                bot.edit_message_reply_markup(chat_id=message.chat_id,
-                                              message_id=message.message_id,
-                                              *args,
-                                              **kwargs)
+            bot.edit_message_reply_markup(chat_id=message.chat_id,
+                                          message_id=message.message_id,
+                                          *args,
+                                          **kwargs)
 
         Note:
             You can only edit messages that the bot sent itself,
@@ -620,18 +652,16 @@ class Message(TelegramObject):
         Returns:
             :class:`telegram.Message`: On success, instance representing the edited message.
         """
-
         return self.bot.edit_message_reply_markup(
             chat_id=self.chat_id, message_id=self.message_id, *args, **kwargs)
 
     def delete(self, *args, **kwargs):
-        """
-        Shortcut for::
+        """Shortcut for::
 
-                 bot.delete_message(chat_id=message.chat_id,
-                                    message_id=message.message_id,
-                                    *args,
-                                    **kwargs)
+             bot.delete_message(chat_id=message.chat_id,
+                                message_id=message.message_id,
+                                *args,
+                                **kwargs)
 
         Returns:
             :obj:`bool`: On success, ``True`` is returned.
@@ -641,8 +671,7 @@ class Message(TelegramObject):
             chat_id=self.chat_id, message_id=self.message_id, *args, **kwargs)
 
     def parse_entity(self, entity):
-        """
-        Returns the text from a given :class:`telegram.MessageEntity`.
+        """Returns the text from a given :class:`telegram.MessageEntity`.
 
         Note:
             This method is present because Telegram calculates the offset and length in
@@ -655,8 +684,8 @@ class Message(TelegramObject):
 
         Returns:
             str: The text of the given entity
-        """
 
+        """
         # Is it a narrow build, if so we don't need to convert
         if sys.maxunicode == 0xffff:
             return self.text[entity.offset:entity.offset + entity.length]
@@ -687,8 +716,8 @@ class Message(TelegramObject):
         Returns:
             Dict[:class:`telegram.MessageEntity`, :obj:`str`]: A dictionary of entities mapped to
             the text that belongs to them, calculated based on UTF-16 codepoints.
-        """
 
+        """
         if types is None:
             types = MessageEntity.ALL_TYPES
 
@@ -697,23 +726,13 @@ class Message(TelegramObject):
             for entity in self.entities if entity.type in types
         }
 
-    @property
-    def text_html(self):
-        """
-        Creates an HTML-formatted string from the markup entities found in the message.
-
-        Use this if you want to retrieve the message text with the entities formatted as HTML.
-
-        Returns:
-            :obj:`str`: Message text with entities formatted as HTML.
-        """
-
+    def _text_html(self, urled=False):
         entities = self.parse_entities()
         message_text = self.text
         if not sys.maxunicode == 0xffff:
             message_text = message_text.encode('utf-16-le')
 
-        markdown_text = ''
+        html_text = ''
         last_offset = 0
 
         for entity, text in sorted(entities.items(), key=(lambda item: item[0].offset)):
@@ -721,7 +740,7 @@ class Message(TelegramObject):
 
             if entity.type == MessageEntity.TEXT_LINK:
                 insert = '<a href="{}">{}</a>'.format(entity.url, text)
-            elif entity.type == MessageEntity.URL:
+            elif (entity.type == MessageEntity.URL) and urled:
                 insert = '<a href="{0}">{0}</a>'.format(text)
             elif entity.type == MessageEntity.BOLD:
                 insert = '<b>' + text + '</b>'
@@ -735,30 +754,46 @@ class Message(TelegramObject):
                 insert = text
 
             if sys.maxunicode == 0xffff:
-                markdown_text += escape_html(message_text[last_offset:entity.offset]) + insert
+                html_text += escape_html(message_text[last_offset:entity.offset]) + insert
             else:
-                markdown_text += escape_html(message_text[last_offset * 2:entity.offset * 2]
-                                             .decode('utf-16-le')) + insert
+                html_text += escape_html(message_text[last_offset * 2:entity.offset * 2]
+                                         .decode('utf-16-le')) + insert
 
             last_offset = entity.offset + entity.length
 
         if sys.maxunicode == 0xffff:
-            markdown_text += escape_html(message_text[last_offset:])
+            html_text += escape_html(message_text[last_offset:])
         else:
-            markdown_text += escape_html(message_text[last_offset * 2:].decode('utf-16-le'))
-        return markdown_text
+            html_text += escape_html(message_text[last_offset * 2:].decode('utf-16-le'))
+        return html_text
 
     @property
-    def text_markdown(self):
-        """
-        Creates an Markdown-formatted string from the markup entities found in the message.
+    def text_html(self):
+        """Creates an HTML-formatted string from the markup entities found in the message.
 
-        Use this if you want to retrieve the message text with the entities formatted as Markdown.
+        Use this if you want to retrieve the message text with the entities formatted as HTML in
+        the same way the original message was formatted.
 
         Returns:
-            :obj:`str`: Message text with entities formatted as Markdown.
-        """
+            :obj:`str`: Message text with entities formatted as HTML.
 
+        """
+        return self._text_html(urled=False)
+
+    @property
+    def text_html_urled(self):
+        """Creates an HTML-formatted string from the markup entities found in the message.
+
+        Use this if you want to retrieve the message text with the entities formatted as HTML.
+        This also formats :attr:`telegram.MessageEntity.URL` as a hyperlink.
+
+        Returns:
+            :obj:`str`: Message text with entities formatted as HTML.
+
+        """
+        return self._text_html(urled=True)
+
+    def _text_markdown(self, urled=False):
         entities = self.parse_entities()
         message_text = self.text
         if not sys.maxunicode == 0xffff:
@@ -772,7 +807,7 @@ class Message(TelegramObject):
 
             if entity.type == MessageEntity.TEXT_LINK:
                 insert = '[{}]({})'.format(text, entity.url)
-            elif entity.type == MessageEntity.URL:
+            elif (entity.type == MessageEntity.URL) and urled:
                 insert = '[{0}]({0})'.format(text)
             elif entity.type == MessageEntity.BOLD:
                 insert = '*' + text + '*'
@@ -798,6 +833,32 @@ class Message(TelegramObject):
             markdown_text += escape_markdown(message_text[last_offset * 2:].decode('utf-16-le'))
         return markdown_text
 
+    @property
+    def text_markdown(self):
+        """Creates an Markdown-formatted string from the markup entities found in the message.
+
+        Use this if you want to retrieve the message text with the entities formatted as Markdown
+        in the same way the original message was formatted.
+
+        Returns:
+            :obj:`str`: Message text with entities formatted as Markdown.
+
+        """
+        return self._text_markdown(urled=False)
+
+    @property
+    def text_markdown_urled(self):
+        """Creates an Markdown-formatted string from the markup entities found in the message.
+
+        Use this if you want to retrieve the message text with the entities formatted as Markdown.
+        This also formats :attr:`telegram.MessageEntity.URL` as a hyperlink.
+
+        Returns:
+            :obj:`str`: Message text with entities formatted as Markdown.
+
+        """
+        return self._text_markdown(urled=True)
+
     @property
     def new_chat_member(self):
         """Deprecated"""
